@@ -10,14 +10,14 @@ Test::BinaryData - compare two things, give hex dumps if they differ
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
- $Id: /my/cs/projects/Test-BinaryData/trunk/lib/Test/BinaryData.pm 32188 2007-07-27T22:08:19.378295Z rjbs  $
+ $Id: /my/cs/projects/Test-BinaryData/trunk/lib/Test/BinaryData.pm 32199 2007-07-28T18:49:17.909294Z rjbs  $
 
 =cut
 
 use vars qw($VERSION);
-$VERSION = '0.004';
+$VERSION = '0.005';
 
 =head1 SYNOPSIS
 
@@ -80,6 +80,16 @@ differences.  Here, we'll just look for two:
   # 20435220616e64206e6f  CR and no = 20435220616e64206e6f  CR and no
   # ...
 
+=head1 WARNINGS
+
+It turns out that the C<substr> provided by F<bytes.pm> is busted.  This hangs:
+
+  bytes::substr("foo", 1, 0);
+
+What the heck?!  So, on 5.6, we use the core substr and length routines, which
+means that wide character support is busted.  This shouldn't be too surprising:
+Unicode on 5.6 is busted in general.
+
 =cut
 
 use Carp ();
@@ -88,7 +98,12 @@ require Exporter;
 @Test::BinaryData::ISA = qw(Exporter);
 @Test::BinaryData::EXPORT = qw(is_binary);
 
-require bytes;
+BEGIN {
+  if ($] >= 5.008) {
+    require bytes;
+    bytes->import;
+  }
+}
 
 my $Test = Test::Builder->new;
 
@@ -161,7 +176,7 @@ sub is_binary {
 
   $Test->ok(0, $comment);
 
-  my $max_length = (sort map { bytes::length($_) } $got, $expected)[1];
+  my $max_length = (sort map { length($_) } $got, $expected)[1];
 
   $Test->diag(
     sprintf "%-${hw}s %-${aw}s   %-${hw}s %-${aw}s",
@@ -175,30 +190,30 @@ sub is_binary {
       last CHUNK;
     }
 
-    my $g_substr = bytes::substr($got,      $pos, $aw);
-    my $e_substr = bytes::substr($expected, $pos, $aw);
+    my $g_substr = substr($got,      $pos, $aw);
+    my $e_substr = substr($expected, $pos, $aw);
 
     my $eq = $g_substr eq $e_substr;
 
     my $g_hex =
       join q{},
-      map { sprintf '%02x', ord(bytes::substr($g_substr, $_, 1)) }
-      0 .. bytes::length($g_substr) - 1;
+      map { sprintf '%02x', ord(substr($g_substr, $_, 1)) }
+      0 .. length($g_substr) - 1;
 
     my $e_hex =
       join q{},
-      map { sprintf '%02x', ord(bytes::substr($e_substr, $_, 1)) }
-      0 .. bytes::length($e_substr) - 1;
+      map { sprintf '%02x', ord(substr($e_substr, $_, 1)) }
+      0 .. length($e_substr) - 1;
 
     for my $str ($g_substr, $e_substr) {
-      for my $pos (0 .. bytes::length($str) - 1) {
-        my $c = bytes::substr($str, $pos, 1);
-        bytes::substr($str, $pos, 1, q{.}) if ord($c) < 0x20 or ord($c) > 0x7e;
+      for my $pos (0 .. length($str) - 1) {
+        my $c = substr($str, $pos, 1);
+        substr($str, $pos, 1, q{.}) if ord($c) < 0x20 or ord($c) > 0x7e;
       }
     }
 
     $_ = sprintf "%-${aw}s", $_ for ($g_substr, $e_substr);
-    $_ .= q{-} x ($hw - bytes::length($_)) for ($g_hex, $e_hex);
+    $_ .= q{-} x ($hw - length($_)) for ($g_hex, $e_hex);
 
     $Test->diag(
       "$g_hex $g_substr",
